%% Recog
%% Read two images and try to match interest points to detect an object
function stereo

[x1,map]=imread('cam1.tif'); % Input image
[x2,map]=imread('cam2.tif'); % Pattern to be detected
I1 = double(x1(:,:,1));
I2 = double(x2(:,:,1));
%% subsample
step = 4;
I1 = I1(1:step:end,1:step:end);
I2 = I2(1:step:end,1:step:end);
%% Compute Harris detector from both imput image and reference pattern
Pts1 = harris(I1);
Pts2 = harris(I2);
%% ....Pts1 and Pts2 are coordinates of Harris points into I1 and I2
%% Compute descriptors from each Harris point
z1=descriptor(I1,Pts1,11);
z2=descriptor(I2,Pts2,11);
%% Compute interdistance matrix between the two sets of descriptors
D2 = distSqrd(z1,z2);
%% Match descriptors
indl=[];
indc=[];
seuil = 5.*min(D2(:));
[indl, indc] = match(D2,seuil,indl,indc);
%% ... P1 are matched points coordinates into reference pattern
P1 = Pts1(indl,:);
P2 = Pts2(indc,:);
%% Modify to estimate 3D points from the set of matches
% ...
%% Basic Display
subplot(1,2,1)
I = [I1 zeros(size(I1,1),size(I2,2)) ; zeros(size(I2,1),size(I1,2))  I2]
image(I);
colormap(gray(256));
hold on 
plot(Pts1(:,1),Pts1(:,2),'+r');
plot(size(I1,2)+Pts2(:,1),size(I1,1)+Pts2(:,2),'+r');
title('Images and matches')
% Green Points denote matched points between reference image and pattern
for n=1:size(P1,1)
    plot([P1(n,1);size(I1,2)+P2(n,1)],[P1(n,2);size(I1,1)+P2(n,2)]);
end
hold off
%% DISPLAY 3D set of point using plot3
subplot(1,2,1)
% ...



%% Return the (2n+1)x(2n+1) gray level ROIs centered on Pts (Nbx2)
function z=descriptor(Im,Pts,n)
[nl,nc]=size(Pts);
z = zeros(nl,(2*n+1)^2);
for l = 1:nl
     tmp = Im(Pts(l,2)-n:Pts(l,2)+n,Pts(l,1)-n:Pts(l,1)+n);
     z(l,:) = [tmp(:)]';
end

%% Compute interdistance matrix
function D2 = distSqrd(X,Y)
    nx      = size(X,1);
    ny      = size(Y,1);
    D2      = sum(X.^2,2)*ones(1,ny) + ones(nx,1)*sum(Y.^2,2)' - 2*(X*Y');
    
    
%% Matching    
function [indl, indc] = match(M1,th,indl,indc)
% brut force matching 
% Min: input distance matrix
% th: distance threshold
% indl: selected lines (initialized to null vector)
% indc: selected columns (initialized to null vector)
[val,ind]=min(M1);
[va2,indctmp]=min(val);
if va2<th
    indl = [ indl ind(indctmp)];
    indc = [ indc indctmp];
    M1(ind(indc),:)=th+1;
    M1(:,indctmp)=th+1;    
    [indl, indc] = match(M1,th,indl,indc);
end
